import React, { useState, useEffect } from 'react';
import { useNavigate } from 'react-router-dom';
import { useDispatch, useSelector } from 'react-redux';
import { showNotification } from '../../store/notification.slice';
import { fetchAvailableDatasets, fetchChats, createConversation } from '../../store/chat/chat.thunks';
import SvgIcon from '../../components/SvgIcon';

export default function NewChat() {
    const navigate = useNavigate();
    const dispatch = useDispatch();
    const [selectedDatasetIds, setSelectedDatasetIds] = useState([]);
    const [isNavigating, setIsNavigating] = useState(false);

    // 从 Redux store 获取可用知识库数据
    const datasets = useSelector((state) => state.chat.availableDatasets.items || []);
    const isLoading = useSelector((state) => state.chat.availableDatasets.status === 'loading');
    const error = useSelector((state) => state.chat.availableDatasets.error);

    // 获取聊天历史记录
    const chatHistory = useSelector((state) => state.chat.history.items || []);
    const chatHistoryLoading = useSelector((state) => state.chat.history.status === 'loading');
    const chatCreationStatus = useSelector((state) => state.chat.sendMessage?.status);

    // 获取可用知识库列表和聊天历史
    useEffect(() => {
        dispatch(fetchAvailableDatasets());
        dispatch(fetchChats({ page: 1, page_size: 50 }));
    }, [dispatch]);

    // 监听错误状态
    useEffect(() => {
        if (error) {
            dispatch(
                showNotification({
                    message: `获取可用知识库列表失败: ${error}`,
                    type: 'danger',
                })
            );
        }
    }, [error, dispatch]);

    // 处理知识库选择（切换选择状态）
    const handleToggleKnowledgeBase = (dataset) => {
        if (isNavigating) return; // 如果正在导航中，阻止操作

        setSelectedDatasetIds((prev) => {
            // 检查是否已经选中
            const isSelected = prev.includes(dataset.id);

            if (isSelected) {
                // 如果已选中，则从数组中移除
                return prev.filter((id) => id !== dataset.id);
            } else {
                // 如果未选中，则添加到数组中
                return [...prev, dataset.id];
            }
        });
    };

    // 开始聊天
    const handleStartChat = async () => {
        if (selectedDatasetIds.length === 0) {
            dispatch(
                showNotification({
                    message: '请至少选择一个知识库',
                    type: 'warning',
                })
            );
            return;
        }

        if (isNavigating) return; // 防止重复点击

        try {
            setIsNavigating(true);

            // 打印调试信息
            console.log('选中的知识库ID:', selectedDatasetIds);

            // 检查是否已存在包含所有选中知识库的聊天记录
            // 注意：这里的逻辑简化了，实际可能需要更复杂的匹配算法
            const existingChat = chatHistory.find((chat) => {
                // 检查聊天记录中的知识库是否完全匹配当前选择
                if (chat.datasets && Array.isArray(chat.datasets)) {
                    const chatDatasetIds = chat.datasets.map((ds) => ds.id);
                    return (
                        chatDatasetIds.length === selectedDatasetIds.length &&
                        selectedDatasetIds.every((id) => chatDatasetIds.includes(id))
                    );
                }

                // 兼容旧格式
                if (chat.dataset_id_list && Array.isArray(chat.dataset_id_list)) {
                    const formattedSelectedIds = selectedDatasetIds.map((id) => id.replace(/-/g, ''));
                    return (
                        chat.dataset_id_list.length === formattedSelectedIds.length &&
                        formattedSelectedIds.every((id) => chat.dataset_id_list.includes(id))
                    );
                }

                return false;
            });

            if (existingChat) {
                // 找到现有聊天记录，导航到该聊天页面
                // 使用所有知识库ID作为URL参数，以逗号分隔
                const knowledgeBaseIdsParam = selectedDatasetIds.join(',');
                console.log(
                    `找到现有聊天记录，直接导航到 /chat/${knowledgeBaseIdsParam}/${existingChat.conversation_id}`
                );
                navigate(`/chat/${knowledgeBaseIdsParam}/${existingChat.conversation_id}`);
            } else {
                // 没有找到现有聊天记录，创建新的聊天
                console.log(`未找到现有聊天记录，创建新会话，选中的知识库ID: ${selectedDatasetIds.join(', ')}`);

                try {
                    // 调用createConversation创建新会话（不发送消息）
                    const response = await dispatch(
                        createConversation({
                            dataset_id_list: selectedDatasetIds,
                        })
                    ).unwrap();

                    console.log('创建会话响应:', response);

                    if (response && response.conversation_id) {
                        // 使用所有知识库ID作为URL参数，以逗号分隔
                        const knowledgeBaseIdsParam = selectedDatasetIds.join(',');
                        console.log(`创建会话成功，导航到 /chat/${knowledgeBaseIdsParam}/${response.conversation_id}`);
                        navigate(`/chat/${knowledgeBaseIdsParam}/${response.conversation_id}`);
                    } else {
                        throw new Error('未能获取会话ID：' + JSON.stringify(response));
                    }
                } catch (apiError) {
                    // 专门处理API调用错误
                    console.error('API调用失败:', apiError);
                    throw new Error(`API调用失败: ${apiError.message || '未知错误'}`);
                }
            }
        } catch (error) {
            console.error('导航或创建聊天失败:', error);

            // 添加更详细的错误日志
            if (error.stack) {
                console.error('错误堆栈:', error.stack);
            }

            // 显示更友好的错误消息
            dispatch(
                showNotification({
                    message: `创建聊天失败: ${error.message || '请重试'}`,
                    type: 'danger',
                })
            );
        } finally {
            setIsNavigating(false);
        }
    };

    // 渲染加载状态
    if (isLoading || chatHistoryLoading) {
        return (
            <div className='container-fluid px-4 py-5 text-center'>
                <div className='spinner-border' role='status'>
                    <span className='visually-hidden'>加载中...</span>
                </div>
            </div>
        );
    }

    return (
        <div className='container-fluid px-4 py-5'>
            {/* 导航中的遮罩层 */}
            {isNavigating && (
                <div
                    className='position-fixed top-0 start-0 w-100 h-100 d-flex justify-content-center align-items-center'
                    style={{
                        backgroundColor: 'rgba(255, 255, 255, 0.7)',
                        zIndex: 1050,
                    }}
                >
                    <div className='text-center'>
                        <div className='spinner-border mb-2' role='status'>
                            <span className='visually-hidden'>加载中...</span>
                        </div>
                        <div>正在加载聊天界面...</div>
                    </div>
                </div>
            )}

            <div className='d-flex justify-content-between align-items-center mb-4'>
                <h4 className='m-0'>选择知识库开始聊天</h4>
                <button
                    className='btn btn-dark'
                    onClick={handleStartChat}
                    disabled={selectedDatasetIds.length === 0 || isNavigating}
                >
                    {isNavigating ? (
                        <>
                            <span
                                className='spinner-border spinner-border-sm me-2'
                                role='status'
                                aria-hidden='true'
                            ></span>
                            加载中...
                        </>
                    ) : (
                        <>开始聊天 {selectedDatasetIds.length > 0 && `(${selectedDatasetIds.length})`}</>
                    )}
                </button>
            </div>

            {selectedDatasetIds.length > 0 && (
                <div className='alert alert-dark mb-4'>已选择 {selectedDatasetIds.length} 个知识库</div>
            )}

            <div className='row row-cols-1 row-cols-md-2 row-cols-lg-3 g-4 justify-content-center'>
                {datasets.length > 0 ? (
                    datasets.map((dataset) => {
                        const isSelected = selectedDatasetIds.includes(dataset.id);
                        return (
                            <div key={dataset.id} className='col'>
                                <div
                                    className={`card h-100 shadow-sm ${!isNavigating ? 'cursor-pointer' : ''} ${
                                        isSelected ? 'border-gray border-2' : 'border-0'
                                    }`}
                                    onClick={() => handleToggleKnowledgeBase(dataset)}
                                    style={{ opacity: isNavigating && !isSelected ? 0.6 : 1 }}
                                >
                                    <div className='card-body'>
                                        <h5 className='card-title d-flex justify-content-between align-items-center'>
                                            {dataset.name}
                                            {isSelected && <SvgIcon className='check-circle text-primary' />}
                                        </h5>
                                        <p className='card-text text-muted'>
                                            {dataset.desc || dataset.description || ''}
                                        </p>
                                        <div className='text-muted small d-flex align-items-center gap-2'>
                                            <span className='d-flex align-items-center gap-1'>
                                                {dataset.department || ''}
                                            </span>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        );
                    })
                ) : (
                    <div className='col-12'>
                        <div className='alert alert-warning'>暂无可访问的知识库，请先申请知识库访问权限</div>
                    </div>
                )}
            </div>
        </div>
    );
}
