import React, { useState, useEffect } from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { formatDate } from '../../../../utils/dateUtils';
import { deleteKnowledgeBaseDocument } from '../../../../store/knowledgeBase/knowledgeBase.thunks';
import DocumentPreviewModal from './DocumentPreviewModal';

/**
 * 知识库文档列表组件
 */
const DocumentList = ({ knowledgeBaseId }) => {
    const dispatch = useDispatch();
    const { items, loading, error } = useSelector((state) => state.knowledgeBase.documents);
    const [previewModalVisible, setPreviewModalVisible] = useState(false);
    const [selectedDocumentId, setSelectedDocumentId] = useState(null);

    // 前端分页状态
    const [currentPage, setCurrentPage] = useState(1);
    const [pageSize, setPageSize] = useState(10);
    const [displayedItems, setDisplayedItems] = useState([]);
    const [totalPages, setTotalPages] = useState(1);

    // 处理分页
    useEffect(() => {
        if (!items || items.length === 0) {
            setDisplayedItems([]);
            setTotalPages(1);
            return;
        }

        // 计算总页数
        const total = Math.ceil(items.length / pageSize);
        setTotalPages(total);

        // 确保当前页有效
        let page = currentPage;
        if (page > total) {
            page = total;
            setCurrentPage(page);
        }

        // 计算当前页显示的项目
        const startIndex = (page - 1) * pageSize;
        const endIndex = Math.min(startIndex + pageSize, items.length);
        setDisplayedItems(items.slice(startIndex, endIndex));

        console.log(`前端分页: 总项目 ${items.length}, 当前页 ${page}/${total}, 显示 ${startIndex + 1}-${endIndex}`);
    }, [items, currentPage, pageSize]);

    // 分页控制
    const handlePageChange = (page) => {
        setCurrentPage(page);
    };

    const handlePrevPage = () => {
        if (currentPage > 1) {
            setCurrentPage(currentPage - 1);
        }
    };

    const handleNextPage = () => {
        if (currentPage < totalPages) {
            setCurrentPage(currentPage + 1);
        }
    };

    // 调试输出
    useEffect(() => {
        console.log('DocumentList 渲染 - 知识库ID:', knowledgeBaseId);
        console.log('文档列表状态:', { items, loading, error });
        console.log('文档列表项数:', items ? items.length : 0);
    }, [knowledgeBaseId, items, loading, error]);

    const handleDeleteDocument = (documentId) => {
        if (window.confirm('确定要删除此文档吗？')) {
            dispatch(
                deleteKnowledgeBaseDocument({
                    knowledge_base_id: knowledgeBaseId,
                    document_id: documentId,
                })
            );
        }
    };

    const handlePreviewDocument = (documentId) => {
        setSelectedDocumentId(documentId);
        setPreviewModalVisible(true);
    };

    const handleClosePreviewModal = () => {
        setPreviewModalVisible(false);
        setSelectedDocumentId(null);
    };

    if (loading) {
        console.log('DocumentList - 加载中...');
        return (
            <div className='text-center py-4'>
                <div className='spinner-border text-primary' role='status'>
                    <span className='visually-hidden'>加载中...</span>
                </div>
            </div>
        );
    }

    if (!items || items.length === 0) {
        console.log('DocumentList - 暂无文档');
        return (
            <div className='text-center py-4 text-muted'>
                <p>暂无文档，请上传文档</p>
            </div>
        );
    }

    console.log('DocumentList - 渲染文档列表', displayedItems);
    return (
        <>
            <div className='table-responsive'>
                <table className='table table-hover'>
                    <thead className='table-light'>
                        <tr>
                            <th scope='col'>文档名称</th>
                            <th scope='col'>创建时间</th>
                            <th scope='col'>更新时间</th>
                            <th scope='col'>操作</th>
                        </tr>
                    </thead>
                    <tbody>
                        {displayedItems.map((doc) => (
                            <tr key={doc.id || doc.document_id}>
                                <td>
                                    <div
                                        className='text-truncate'
                                        style={{ maxWidth: '250px' }}
                                        title={doc.document_name || doc.name}
                                    >
                                        {doc.document_name || doc.name}
                                    </div>
                                </td>
                                <td className='text-nowrap'>{formatDateTime(doc.create_time || doc.created_at)}</td>
                                <td className='text-nowrap'>{formatDateTime(doc.update_time || doc.updated_at)}</td>
                                <td>
                                    <div className='btn-group' role='group'>
                                        <button
                                            className='btn btn-sm btn-outline-dark me-2'
                                            onClick={() => handlePreviewDocument(doc.document_id || doc.id)}
                                        >
                                            预览
                                        </button>
                                        <button
                                            className='btn btn-sm btn-outline-danger'
                                            onClick={() => handleDeleteDocument(doc.document_id || doc.id)}
                                        >
                                            删除
                                        </button>
                                    </div>
                                </td>
                            </tr>
                        ))}
                    </tbody>
                </table>

                {/* 分页控件 */}
                {items.length > 0 && (
                    <div className='d-flex justify-content-between align-items-center mt-3'>
                        <div className='text-muted mb-0'>
                            共 {items.length} 条记录，第 {currentPage}/{totalPages} 页
                        </div>
                        <nav aria-label='文档列表分页'>
                            <ul className='pagination dark-pagination pagination-sm mb-0'>
                                <li className={`page-item ${currentPage === 1 ? 'disabled' : ''}`}>
                                    <button className='page-link' onClick={handlePrevPage}>
                                        上一页
                                    </button>
                                </li>
                                {Array.from({ length: totalPages }, (_, i) => i + 1).map((page) => (
                                    <li key={page} className={`page-item ${page === currentPage ? 'active' : ''}`}>
                                        <button className='page-link' onClick={() => handlePageChange(page)}>
                                            {page}
                                        </button>
                                    </li>
                                ))}
                                <li className={`page-item ${currentPage === totalPages ? 'disabled' : ''}`}>
                                    <button className='page-link' onClick={handleNextPage}>
                                        下一页
                                    </button>
                                </li>
                            </ul>
                        </nav>
                    </div>
                )}
            </div>

            <DocumentPreviewModal
                show={previewModalVisible}
                documentId={selectedDocumentId}
                knowledgeBaseId={knowledgeBaseId}
                onClose={handleClosePreviewModal}
            />
        </>
    );
};

// Helper function to format date string
const formatDateTime = (dateString) => {
    if (!dateString) return '-';

    // If the utility function exists, use it, otherwise format manually
    try {
        return formatDate(dateString);
    } catch (error) {
        const date = new Date(dateString);
        return date.toLocaleString();
    }
};

export default DocumentList;
