import React, { useState, useRef, useEffect } from 'react';
import { useDispatch } from 'react-redux';
import { showNotification } from '../../../../store/notification.slice';
import SvgIcon from '../../../../components/SvgIcon';

/**
 * 用户权限管理组件
 */
const UserPermissionsManager = ({ knowledgeBase }) => {
    const dispatch = useDispatch();

    // State for pagination
    const [currentPage, setCurrentPage] = useState(1);
    const usersPerPage = 10;

    // State for edit modal
    const [showEditModal, setShowEditModal] = useState(false);
    const [editUser, setEditUser] = useState(null);

    // State for add user modal
    const [showAddUserModal, setShowAddUserModal] = useState(false);
    const [newUser, setNewUser] = useState({
        username: '',
        email: '',
        permissionType: '只读',
        accessDuration: '一个月',
    });

    // State for batch operations
    const [selectedUsers, setSelectedUsers] = useState([]);
    const [selectAll, setSelectAll] = useState(false);
    const [showBatchDropdown, setShowBatchDropdown] = useState(false);
    const batchDropdownRef = useRef(null);

    // State for batch edit modal
    const [showBatchEditModal, setShowBatchEditModal] = useState(false);
    const [batchEditData, setBatchEditData] = useState({
        permissionType: '只读',
        accessDuration: '一个月',
    });

    // Form errors
    const [formErrors, setFormErrors] = useState({});

    // Mock data for users with permissions
    const [users, setUsers] = useState([
        {
            id: '1001',
            username: '张三',
            email: 'zhang@abc.com',
            permissionType: '只读',
            accessDuration: '一个月',
        },
        {
            id: '1002',
            username: '李四',
            email: 'li@abc.com',
            permissionType: '完全访问',
            accessDuration: '永久',
        },
        {
            id: '1003',
            username: '王五',
            email: 'wang@abc.com',
            permissionType: '只读',
            accessDuration: '三个月',
        },
        {
            id: '1004',
            username: '赵六',
            email: 'zhao@abc.com',
            permissionType: '完全访问',
            accessDuration: '六个月',
        },
        {
            id: '1005',
            username: '钱七',
            email: 'qian@abc.com',
            permissionType: '只读',
            accessDuration: '一周',
        },
        {
            id: '1006',
            username: '孙八',
            email: 'sun@abc.com',
            permissionType: '只读',
            accessDuration: '一个月',
        },
        {
            id: '1007',
            username: '周九',
            email: 'zhou@abc.com',
            permissionType: '完全访问',
            accessDuration: '永久',
        },
        {
            id: '1008',
            username: '吴十',
            email: 'wu@abc.com',
            permissionType: '只读',
            accessDuration: '三个月',
        },
    ]);

    // Get current users for pagination
    const indexOfLastUser = currentPage * usersPerPage;
    const indexOfFirstUser = indexOfLastUser - usersPerPage;
    const currentUsers = users.slice(indexOfFirstUser, indexOfLastUser);
    const totalPages = Math.ceil(users.length / usersPerPage);

    // Handle click outside batch dropdown
    useEffect(() => {
        function handleClickOutside(event) {
            if (batchDropdownRef.current && !batchDropdownRef.current.contains(event.target)) {
                setShowBatchDropdown(false);
            }
        }

        document.addEventListener('mousedown', handleClickOutside);
        return () => {
            document.removeEventListener('mousedown', handleClickOutside);
        };
    }, []);

    // Handle edit user permissions
    const handleEditUser = (user) => {
        setEditUser({ ...user });
        setFormErrors({});
        setShowEditModal(true);
    };

    // Handle input change in edit modal
    const handleEditInputChange = (e) => {
        const { name, value } = e.target;
        setEditUser((prev) => ({
            ...prev,
            [name]: value,
        }));

        // Clear error if exists
        if (formErrors[name]) {
            setFormErrors((prev) => ({
                ...prev,
                [name]: '',
            }));
        }
    };

    // Handle input change in add user modal
    const handleAddUserInputChange = (e) => {
        const { name, value } = e.target;
        setNewUser((prev) => ({
            ...prev,
            [name]: value,
        }));

        // Clear error if exists
        if (formErrors[name]) {
            setFormErrors((prev) => ({
                ...prev,
                [name]: '',
            }));
        }
    };

    // Handle input change in batch edit modal
    const handleBatchEditInputChange = (e) => {
        const { name, value } = e.target;
        setBatchEditData((prev) => ({
            ...prev,
            [name]: value,
        }));
    };

    // Validate edit form
    const validateEditForm = () => {
        const errors = {};

        if (!editUser.username.trim()) {
            errors.username = '请输入用户名';
        }

        if (!editUser.email.trim()) {
            errors.email = '请输入邮箱';
        } else if (!/\S+@\S+\.\S+/.test(editUser.email)) {
            errors.email = '请输入有效的邮箱地址';
        }

        setFormErrors(errors);
        return Object.keys(errors).length === 0;
    };

    // Validate add user form
    const validateAddUserForm = () => {
        const errors = {};

        if (!newUser.username.trim()) {
            errors.username = '请输入用户名';
        }

        if (!newUser.email.trim()) {
            errors.email = '请输入邮箱';
        } else if (!/\S+@\S+\.\S+/.test(newUser.email)) {
            errors.email = '请输入有效的邮箱地址';
        }

        setFormErrors(errors);
        return Object.keys(errors).length === 0;
    };

    // Handle save user permissions
    const handleSaveUserPermissions = () => {
        // Validate form
        if (!validateEditForm()) {
            return;
        }

        // Here you would typically call an API to update the user permissions
        console.log('Updating user permissions:', editUser);

        // Update the users array with the edited user
        setUsers((prevUsers) => prevUsers.map((user) => (user.id === editUser.id ? { ...editUser } : user)));

        // Show success notification
        dispatch(showNotification({ message: '用户权限已更新', type: 'success' }));

        // Close modal
        setShowEditModal(false);
    };

    // Handle add new user
    const handleAddUser = () => {
        // Validate form
        if (!validateAddUserForm()) {
            return;
        }

        // Generate a new ID
        const newId = `user-${Date.now()}`;

        // Create new user object
        const userToAdd = {
            id: newId,
            username: newUser.username,
            email: newUser.email,
            permissionType: newUser.permissionType,
            accessDuration: newUser.accessDuration,
        };

        // Add new user to the users array
        setUsers((prevUsers) => [...prevUsers, userToAdd]);

        // Show success notification
        dispatch(showNotification({ message: '用户已添加', type: 'success' }));

        // Reset form and close modal
        setNewUser({
            username: '',
            email: '',
            permissionType: '只读',
            accessDuration: '一个月',
        });
        setShowAddUserModal(false);
    };

    // Handle delete user
    const handleDeleteUser = (userId) => {
        // Here you would typically call an API to delete the user
        console.log('Deleting user:', userId);

        // Remove user from the users array
        setUsers((prevUsers) => prevUsers.filter((user) => user.id !== userId));

        // Remove from selected users if present
        setSelectedUsers((prev) => prev.filter((id) => id !== userId));

        // Show success notification
        dispatch(showNotification({ message: '用户已删除', type: 'success' }));
    };

    // Handle batch delete
    const handleBatchDelete = () => {
        if (selectedUsers.length === 0) return;

        // Here you would typically call an API to delete the selected users
        console.log('Batch deleting users:', selectedUsers);

        // Remove selected users from the users array
        setUsers((prevUsers) => prevUsers.filter((user) => !selectedUsers.includes(user.id)));

        // Reset selection
        setSelectedUsers([]);
        setSelectAll(false);
        setShowBatchDropdown(false);

        // Show success notification
        dispatch(showNotification({ message: `已删除 ${selectedUsers.length} 个用户`, type: 'success' }));
    };

    // Handle batch edit
    const handleBatchEdit = () => {
        if (selectedUsers.length === 0) return;

        // Here you would typically call an API to update the selected users
        console.log('Batch editing users:', selectedUsers, 'with data:', batchEditData);

        // Update selected users in the users array
        setUsers((prevUsers) =>
            prevUsers.map((user) => (selectedUsers.includes(user.id) ? { ...user, ...batchEditData } : user))
        );

        // Close modal
        setShowBatchEditModal(false);
        setShowBatchDropdown(false);

        // Show success notification
        dispatch(showNotification({ message: `已更新 ${selectedUsers.length} 个用户的权限`, type: 'success' }));
    };

    // Handle select all checkbox
    const handleSelectAll = () => {
        if (selectAll) {
            setSelectedUsers([]);
        } else {
            setSelectedUsers(currentUsers.map((user) => user.id));
        }
        setSelectAll(!selectAll);
    };

    // Handle individual user selection
    const handleSelectUser = (userId) => {
        if (selectedUsers.includes(userId)) {
            setSelectedUsers(selectedUsers.filter((id) => id !== userId));
            setSelectAll(false);
        } else {
            setSelectedUsers([...selectedUsers, userId]);
            // Check if all current page users are now selected
            if (selectedUsers.length + 1 === currentUsers.length) {
                setSelectAll(true);
            }
        }
    };

    // Handle page change
    const handlePageChange = (pageNumber) => {
        setCurrentPage(pageNumber);
        // Reset selection when changing pages
        setSelectedUsers([]);
        setSelectAll(false);
    };

    return (
        <div className='card border-0 shadow-sm mt-4'>
            <div className='card-body'>
                <div className='d-flex justify-content-between align-items-center mb-4'>
                    <h5 className='card-title mb-0'>用户权限管理</h5>
                    <div className='d-flex gap-2'>
                        {selectedUsers.length > 0 && (
                            <div className='dropdown' ref={batchDropdownRef}>
                                <button
                                    className='btn btn-outline-primary btn-sm dropdown-toggle'
                                    onClick={() => setShowBatchDropdown(!showBatchDropdown)}
                                >
                                    批量操作 ({selectedUsers.length})
                                </button>
                                {showBatchDropdown && (
                                    <ul className='dropdown-menu show'>
                                        <li>
                                            <button
                                                className='dropdown-item'
                                                onClick={() => {
                                                    setShowBatchEditModal(true);
                                                    setShowBatchDropdown(false);
                                                }}
                                            >
                                                <SvgIcon className='edit' />
                                                <span className='ms-2'>批量修改权限</span>
                                            </button>
                                        </li>
                                        <li>
                                            <button className='dropdown-item text-danger' onClick={handleBatchDelete}>
                                                <SvgIcon className='trash' />
                                                <span className='ms-2'>批量删除</span>
                                            </button>
                                        </li>
                                    </ul>
                                )}
                            </div>
                        )}
                        <button className='btn btn-dark btn-sm' onClick={() => setShowAddUserModal(true)}>
                            <SvgIcon className='plus' />
                            添加用户
                        </button>
                    </div>
                </div>

                <div className='table-responsive'>
                    <table className='table table-hover'>
                        <thead>
                            <tr>
                                <th>
                                    <div className='form-check'>
                                        <input
                                            className='form-check-input'
                                            type='checkbox'
                                            checked={selectAll}
                                            onChange={handleSelectAll}
                                        />
                                    </div>
                                </th>
                                <th>用户名</th>
                                <th>邮箱</th>
                                <th>权限类型</th>
                                <th>访问期限</th>
                                <th>操作</th>
                            </tr>
                        </thead>
                        <tbody>
                            {currentUsers.length > 0 ? (
                                currentUsers.map((user) => (
                                    <tr key={user.id}>
                                        <td>
                                            <div className='form-check'>
                                                <input
                                                    className='form-check-input'
                                                    type='checkbox'
                                                    checked={selectedUsers.includes(user.id)}
                                                    onChange={() => handleSelectUser(user.id)}
                                                />
                                            </div>
                                        </td>
                                        <td>{user.username}</td>
                                        <td>{user.email}</td>
                                        <td>
                                            <span
                                                className={`badge ${
                                                    user.permissionType === '完全访问'
                                                        ? 'bg-success-subtle text-success'
                                                        : 'bg-warning-subtle text-warning'
                                                }`}
                                            >
                                                {user.permissionType}
                                            </span>
                                        </td>
                                        <td>{user.accessDuration}</td>
                                        <td>
                                            <div className='btn-group'>
                                                <button
                                                    className='btn btn-sm text-primary'
                                                    onClick={() => handleEditUser(user)}
                                                >
                                                    <SvgIcon className='edit' />
                                                </button>
                                                <button
                                                    className='btn btn-sm text-danger'
                                                    onClick={() => handleDeleteUser(user.id)}
                                                >
                                                    <SvgIcon className='trash' />
                                                </button>
                                            </div>
                                        </td>
                                    </tr>
                                ))
                            ) : (
                                <tr>
                                    <td colSpan='6' className='text-center'>
                                        暂无用户
                                    </td>
                                </tr>
                            )}
                        </tbody>
                    </table>
                </div>

                {/* Pagination */}
                {totalPages > 1 && (
                    <nav aria-label='Page navigation'>
                        <ul className='pagination justify-content-center'>
                            <li className={`page-item ${currentPage === 1 ? 'disabled' : ''}`}>
                                <button
                                    className='page-link'
                                    onClick={() => handlePageChange(currentPage - 1)}
                                    disabled={currentPage === 1}
                                >
                                    上一页
                                </button>
                            </li>
                            {[...Array(totalPages)].map((_, index) => (
                                <li key={index} className={`page-item ${currentPage === index + 1 ? 'active' : ''}`}>
                                    <button className='page-link' onClick={() => handlePageChange(index + 1)}>
                                        {index + 1}
                                    </button>
                                </li>
                            ))}
                            <li className={`page-item ${currentPage === totalPages ? 'disabled' : ''}`}>
                                <button
                                    className='page-link'
                                    onClick={() => handlePageChange(currentPage + 1)}
                                    disabled={currentPage === totalPages}
                                >
                                    下一页
                                </button>
                            </li>
                        </ul>
                    </nav>
                )}

                {/* Edit User Modal */}
                {showEditModal && (
                    <div className='modal fade show' style={{ display: 'block' }} tabIndex='-1'>
                        <div className='modal-dialog'>
                            <div className='modal-content'>
                                <div className='modal-header'>
                                    <h5 className='modal-title'>编辑用户权限</h5>
                                    <button
                                        type='button'
                                        className='btn-close'
                                        onClick={() => setShowEditModal(false)}
                                    ></button>
                                </div>
                                <div className='modal-body'>
                                    <form>
                                        <div className='mb-3'>
                                            <label htmlFor='username' className='form-label'>
                                                用户名
                                            </label>
                                            <input
                                                type='text'
                                                className={`form-control ${formErrors.username ? 'is-invalid' : ''}`}
                                                id='username'
                                                name='username'
                                                value={editUser.username}
                                                onChange={handleEditInputChange}
                                            />
                                            {formErrors.username && (
                                                <div className='invalid-feedback'>{formErrors.username}</div>
                                            )}
                                        </div>
                                        <div className='mb-3'>
                                            <label htmlFor='email' className='form-label'>
                                                邮箱
                                            </label>
                                            <input
                                                type='email'
                                                className={`form-control ${formErrors.email ? 'is-invalid' : ''}`}
                                                id='email'
                                                name='email'
                                                value={editUser.email}
                                                onChange={handleEditInputChange}
                                            />
                                            {formErrors.email && (
                                                <div className='invalid-feedback'>{formErrors.email}</div>
                                            )}
                                        </div>
                                        <div className='mb-3'>
                                            <label htmlFor='permissionType' className='form-label'>
                                                权限类型
                                            </label>
                                            <select
                                                className='form-select'
                                                id='permissionType'
                                                name='permissionType'
                                                value={editUser.permissionType}
                                                onChange={handleEditInputChange}
                                            >
                                                <option value='只读'>只读</option>
                                                <option value='完全访问'>完全访问</option>
                                            </select>
                                        </div>
                                        <div className='mb-3'>
                                            <label htmlFor='accessDuration' className='form-label'>
                                                访问期限
                                            </label>
                                            <select
                                                className='form-select'
                                                id='accessDuration'
                                                name='accessDuration'
                                                value={editUser.accessDuration}
                                                onChange={handleEditInputChange}
                                            >
                                                <option value='一周'>一周</option>
                                                <option value='一个月'>一个月</option>
                                                <option value='三个月'>三个月</option>
                                                <option value='六个月'>六个月</option>
                                                <option value='永久'>永久</option>
                                            </select>
                                        </div>
                                    </form>
                                </div>
                                <div className='modal-footer'>
                                    <button
                                        type='button'
                                        className='btn btn-secondary'
                                        onClick={() => setShowEditModal(false)}
                                    >
                                        取消
                                    </button>
                                    <button
                                        type='button'
                                        className='btn btn-primary'
                                        onClick={handleSaveUserPermissions}
                                    >
                                        保存
                                    </button>
                                </div>
                            </div>
                        </div>
                    </div>
                )}

                {/* Add User Modal */}
                {showAddUserModal && (
                    <div className='modal fade show' style={{ display: 'block' }} tabIndex='-1'>
                        <div className='modal-dialog'>
                            <div className='modal-content'>
                                <div className='modal-header'>
                                    <h5 className='modal-title'>添加用户</h5>
                                    <button
                                        type='button'
                                        className='btn-close'
                                        onClick={() => setShowAddUserModal(false)}
                                    ></button>
                                </div>
                                <div className='modal-body'>
                                    <form>
                                        <div className='mb-3'>
                                            <label htmlFor='newUsername' className='form-label'>
                                                用户名
                                            </label>
                                            <input
                                                type='text'
                                                className={`form-control ${formErrors.username ? 'is-invalid' : ''}`}
                                                id='newUsername'
                                                name='username'
                                                value={newUser.username}
                                                onChange={handleAddUserInputChange}
                                            />
                                            {formErrors.username && (
                                                <div className='invalid-feedback'>{formErrors.username}</div>
                                            )}
                                        </div>
                                        <div className='mb-3'>
                                            <label htmlFor='newEmail' className='form-label'>
                                                邮箱
                                            </label>
                                            <input
                                                type='email'
                                                className={`form-control ${formErrors.email ? 'is-invalid' : ''}`}
                                                id='newEmail'
                                                name='email'
                                                value={newUser.email}
                                                onChange={handleAddUserInputChange}
                                            />
                                            {formErrors.email && (
                                                <div className='invalid-feedback'>{formErrors.email}</div>
                                            )}
                                        </div>
                                        <div className='mb-3'>
                                            <label htmlFor='newPermissionType' className='form-label'>
                                                权限类型
                                            </label>
                                            <select
                                                className='form-select'
                                                id='newPermissionType'
                                                name='permissionType'
                                                value={newUser.permissionType}
                                                onChange={handleAddUserInputChange}
                                            >
                                                <option value='只读'>只读</option>
                                                <option value='完全访问'>完全访问</option>
                                            </select>
                                        </div>
                                        <div className='mb-3'>
                                            <label htmlFor='newAccessDuration' className='form-label'>
                                                访问期限
                                            </label>
                                            <select
                                                className='form-select'
                                                id='newAccessDuration'
                                                name='accessDuration'
                                                value={newUser.accessDuration}
                                                onChange={handleAddUserInputChange}
                                            >
                                                <option value='一周'>一周</option>
                                                <option value='一个月'>一个月</option>
                                                <option value='三个月'>三个月</option>
                                                <option value='六个月'>六个月</option>
                                                <option value='永久'>永久</option>
                                            </select>
                                        </div>
                                    </form>
                                </div>
                                <div className='modal-footer'>
                                    <button
                                        type='button'
                                        className='btn btn-secondary'
                                        onClick={() => setShowAddUserModal(false)}
                                    >
                                        取消
                                    </button>
                                    <button type='button' className='btn btn-primary' onClick={handleAddUser}>
                                        添加
                                    </button>
                                </div>
                            </div>
                        </div>
                    </div>
                )}

                {/* Batch Edit Modal */}
                {showBatchEditModal && (
                    <div className='modal fade show' style={{ display: 'block' }} tabIndex='-1'>
                        <div className='modal-dialog'>
                            <div className='modal-content'>
                                <div className='modal-header'>
                                    <h5 className='modal-title'>批量修改权限</h5>
                                    <button
                                        type='button'
                                        className='btn-close'
                                        onClick={() => setShowBatchEditModal(false)}
                                    ></button>
                                </div>
                                <div className='modal-body'>
                                    <p>
                                        您正在修改 <strong>{selectedUsers.length}</strong> 个用户的权限
                                    </p>
                                    <form>
                                        <div className='mb-3'>
                                            <label htmlFor='batchPermissionType' className='form-label'>
                                                权限类型
                                            </label>
                                            <select
                                                className='form-select'
                                                id='batchPermissionType'
                                                name='permissionType'
                                                value={batchEditData.permissionType}
                                                onChange={handleBatchEditInputChange}
                                            >
                                                <option value='只读'>只读</option>
                                                <option value='完全访问'>完全访问</option>
                                            </select>
                                        </div>
                                        <div className='mb-3'>
                                            <label htmlFor='batchAccessDuration' className='form-label'>
                                                访问期限
                                            </label>
                                            <select
                                                className='form-select'
                                                id='batchAccessDuration'
                                                name='accessDuration'
                                                value={batchEditData.accessDuration}
                                                onChange={handleBatchEditInputChange}
                                            >
                                                <option value='一周'>一周</option>
                                                <option value='一个月'>一个月</option>
                                                <option value='三个月'>三个月</option>
                                                <option value='六个月'>六个月</option>
                                                <option value='永久'>永久</option>
                                            </select>
                                        </div>
                                    </form>
                                </div>
                                <div className='modal-footer'>
                                    <button
                                        type='button'
                                        className='btn btn-secondary'
                                        onClick={() => setShowBatchEditModal(false)}
                                    >
                                        取消
                                    </button>
                                    <button type='button' className='btn btn-primary' onClick={handleBatchEdit}>
                                        保存
                                    </button>
                                </div>
                            </div>
                        </div>
                    </div>
                )}

                {/* Modal backdrop */}
                {(showEditModal || showAddUserModal || showBatchEditModal) && (
                    <div className='modal-backdrop fade show'></div>
                )}
            </div>
        </div>
    );
};

export default UserPermissionsManager;
